@@ -19,7 +19,6 @@ char *fake = N_("Absolute value");
 char *fake = N_("Replace very small number with zero");
 char *fake = N_("Return the fractional part of a number");
 char *fake = N_("Division w/o remainder");
-char *fake = N_("Check if argument is a gaussian integer");
 char *fake = N_("Return the sign (-1,0,1)");
 char *fake = N_("Logarithm of any base (calls DiscreteLog if in modulo mode), if base is not given, e is used");
 char *fake = N_("The arccos (inverse cos) function");
@@ -130,10 +129,8 @@ char *fake = N_("Get the LU decomposition of A and store the result in the L and
 char *fake = N_("Get the i-j minor of a matrix");
 char *fake = N_("Return the columns that are not the pivot columns of a matrix");
 char *fake = N_("Get the p Norm (or 2 Norm if no p is supplied) of a vector");
-char *fake = N_("Get the nullspace of a matrix");
 char *fake = N_("Get the nullity of a matrix");
 char *fake = N_("Get the orthogonal complement of the columnspace");
-char *fake = N_("Return pivot columns of a matrix, that is columns which have a leading 1 in rref form, also returns the row where they occur");
 char *fake = N_("Projection of vector v onto subspace W given a sesquilinear form B (if not given use hermitian product)");
 char *fake = N_("Get the QR decomposition of A, returns R and Q can be a reference");
 char *fake = N_("Get the rank of a matrix");
