@@ -1,11 +1,13 @@
 /* GENIUS Calculator
- * Copyright (C) 1997-2004 Jiri (George) Lebl
+ * Copyright (C) 1997-2007 Jiri (George) Lebl
  *
  * Author: Jiri (George) Lebl
  *
- * This program is free software; you can redistribute it and/or modify
+ * This file is part of Genius.
+ *
+ * Genius is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,9 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the  Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
- * USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef EVAL_H
@@ -210,6 +210,11 @@ gboolean gel_mod_integer_rational (mpw_t num, mpw_t mod);
 
 mpw_ptr gel_find_pre_function_modulo (GelCtx *ctx);
 
+
+/*compare nodes, return TRUE if equal
+  makes them the same type as a side effect*/
+gboolean gel_eqlnodes (GelETree *l, GelETree *r);
+
 #define GET_ABCDE(n,a,b,c,d,e) { \
 	(a) = (n)->op.args; \
 	(b) = (n)->op.args->any.next; \
@@ -264,6 +269,9 @@ void _gel_make_free_trees (void);
 }
 #endif
 
+extern void (*_gel_tree_limit_hook)(void);
+void gel_test_max_nodes_again (void);
+
 extern GelEFunc *_internal_ln_function;
 extern GelEFunc *_internal_exp_function;
 
