@@ -1,5 +1,5 @@
 /* GENIUS Calculator
- * Copyright (C) 2004 George Lebl
+ * Copyright (C) 2004-2007 George Lebl
  *
  * Author: George Lebl
  *
@@ -25,14 +25,16 @@
 typedef struct {
 	gboolean error_box;
 	gboolean info_box;
+	gboolean blinking_cursor;
 	int scrollback;
 	char *font;
 	gboolean black_on_white;
+	gboolean output_remember; /* remember output settings */
+	gboolean precision_remember; /* remember precision setting */
 } GeniusSetup;
 
 extern GeniusSetup genius_setup;
 extern GtkWidget *genius_window;
-extern gboolean interrupted;
 extern GHashTable *uncompiled;
 extern calcstate_t calcstate;
 
